/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SCREEN_CONNECTOR_REMOTE_SURFACE_EVAS_IMAGE_INTERNAL_H_
#define SCREEN_CONNECTOR_REMOTE_SURFACE_EVAS_IMAGE_INTERNAL_H_

#include <memory>
#include <string>

#include "screen_connector_remote_surface_evas/evas_object.h"

namespace screen_connector {

class Image : public EvasObject {
 public:
  class IEventListener {
   public:
    virtual void OnMouseIn(const EvasObject& obj, void* eventInfo) = 0;
    virtual void OnMouseOut(const EvasObject& obj, void* eventInfo) = 0;
    virtual void OnMouseDown(const EvasObject& obj, void* eventInfo) = 0;
    virtual void OnMouseUp(const EvasObject& obj, void* eventInfo) = 0;
    virtual void OnTouchMultiDown(const EvasObject& obj, void* eventInfo) = 0;
    virtual void OnTouchMultiUp(const EvasObject& obj, void* eventInfo) = 0;
    virtual void OnMouseMove(const EvasObject& obj, void* eventInfo) = 0;
    virtual void OnMouseWheel(const EvasObject& obj, void* eventInfo) = 0;
    virtual void OnMove(const EvasObject& obj, void* eventInfo) = 0;
    virtual void OnShow(const EvasObject& obj, void* eventInfo) = 0;
    virtual void OnHide(const EvasObject& obj, void* eventInfo) = 0;
    virtual void OnResize(const EvasObject& obj, void* eventInfo) = 0;
  };

  Image(Evas_Object* raw, IEventListener* listener, const std::string& instId,
             int pid, void* tag);
  virtual ~Image();

  virtual void Update(std::shared_ptr<WlBuffer> tbm);

 private:
  static void MouseInCB(void* data, Evas* e, Evas_Object* obj, void* eventInfo);
  static void MouseOutCB(void* data, Evas* e, Evas_Object* obj,
                         void* eventInfo);
  static void MouseDownCB(void* data, Evas* e, Evas_Object* obj,
                          void* eventInfo);
  static void MouseUpCB(void* data, Evas* e, Evas_Object* obj, void* eventInfo);
  static void TouchMultiDownCB(void* data, Evas* e, Evas_Object* obj,
                               void* eventInfo);
  static void TouchMultiUpCB(void* data, Evas* e, Evas_Object* obj,
                             void* eventInfo);
  static void MouseMoveCB(void* data, Evas* e, Evas_Object* obj,
                          void* eventInfo);
  static void MouseWheelCB(void* data, Evas* e, Evas_Object* obj,
                           void* eventInfo);
  static void ShowCB(void* data, Evas* e, Evas_Object* obj, void* eventInfo);
  static void HideCB(void* data, Evas* e, Evas_Object* obj, void* eventInfo);
  static void ResizeCB(void* data, Evas* e, Evas_Object* obj, void* eventInfo);
  static void MoveCB(void* data, Evas* e, Evas_Object* obj, void* eventInfo);

 private:
  IEventListener* listener_;
  std::shared_ptr<WlBuffer> prev_buf_;
};

}  // namespace screen_connector

#endif  // SCREEN_CONNECTOR_REMOTE_SURFACE_EVAS_IMAGE_INTERNAL_H_
