/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef SCREEN_CONNECTOR_REMOTE_SURFACE_EVAS_REMOTE_SURFACE_MIXED_INTERNAL_H_
#define SCREEN_CONNECTOR_REMOTE_SURFACE_EVAS_REMOTE_SURFACE_MIXED_INTERNAL_H_

#include <string>
#include <memory>
#include <map>

#include "screen_connector_remote_surface_evas/evas_object.h"
#include "screen_connector_remote_surface_evas/watcher_event_interface.h"
#include "screen_connector_remote_surface/remote_surface.h"
#include "screen_connector_remote_surface_evas/watcher_image_internal.h"
#include "screen_connector_remote_surface/aul_handle_internal.h"

namespace screen_connector {

class RemoteSurfaceMixed : public RemoteSurface {
 public:
  RemoteSurfaceMixed(const std::string& id, RemoteSurface::Type type,
                     std::shared_ptr<EvasObject> viewerWin,
                     IWatcherEvent* listener);
  virtual ~RemoteSurfaceMixed();

 public:
  void SetChangedEventFilter(RemoteSurface::ChangedEventFilter filter) override;
  void OnBufferAdded(const std::string& appId, const std::string& instId,
                     int pid) override;
  void OnBufferRemoved(const std::string& appId, const std::string& instId,
                       int pid) override;
  void OnBufferChanged(int type, std::shared_ptr<WlBuffer> tbm, int fd,
                       uint32_t size, uint32_t time) override;

 private:
  IWatcherEvent* listener_;
  std::unique_ptr<WatcherImage> img_tbm_;
  std::unique_ptr<WatcherImage> img_file_;
  std::shared_ptr<EvasObject> viewer_win_;
};

}  // namespace screen_connector

#endif  // SCREEN_CONNECTOR_REMOTE_SURFACE_EVAS_REMOTE_SURFACE_MIXED_INTERNAL_H_
