/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <dlog.h>
#include <Ecore_Wl2.h>

#include <memory>

#include "screen_connector_remote_surface_evas/remote_surface_watcher.h"
#include "screen_connector_remote_surface_evas/remote_surface_watcher_implementation.h"

#ifdef LOG_TAG
#undef LOG_TAG
#endif

#define LOG_TAG "SC_REMOTE_SURFACE"
#define SC_TOOLKIT_HANDLE_TAG "SC_TOOLKIT_HANDLE_TAG"

namespace screen_connector {

RemoteSurfaceWatcher::Impl::Impl(RemoteSurfaceWatcher* parent,
                                 RemoteSurface::Type type,
                                 std::shared_ptr<EvasObject> viewer_win)
    : viewer_win_(viewer_win),
      aul_handle_(new AulHandle(this, (aul_screen_type_e)type)),
      parent_(parent) {
}

void RemoteSurfaceWatcher::Impl::OnAppAdded(const std::string &app_id,
                                            const std::string& inst_id,
                                            const int pid,
                                            const unsigned int surface_id) {
  auto rs = new RemoteSurfaceMixed(inst_id, RemoteSurface::Type::UI,
      viewer_win_, parent_);
  surfaces_.emplace_back(rs);
}

void RemoteSurfaceWatcher::Impl::OnAppRemoved(const std::string& app_id,
                                              const std::string& inst_id,
                                              const int pid,
                                              const unsigned int surface_id) {
  for (auto& i : surfaces_) {
    if ((i.get())->GetInstId() == inst_id) {
      LOGW("remove %s", inst_id.c_str());
      surfaces_.remove(i);
      break;
    }
  }
}

void RemoteSurfaceWatcher::Impl::OnAppUpdated(const std::string &app_id,
                                              const std::string& inst_id,
                                              const int pid,
                                              const unsigned int surface_id) {
}

RemoteSurfaceWatcher::RemoteSurfaceWatcher(RemoteSurface::Type type,
    std::shared_ptr<EvasObject> viewer_win)
    : impl_(new Impl(this, type, viewer_win)) {
}

RemoteSurfaceWatcher::~RemoteSurfaceWatcher() = default;
RemoteSurfaceWatcher::RemoteSurfaceWatcher(
    RemoteSurfaceWatcher&&) noexcept = default;
RemoteSurfaceWatcher& RemoteSurfaceWatcher::operator=(
    RemoteSurfaceWatcher&&) noexcept = default;

void RemoteSurfaceWatcher::OnWatcherAdded(const std::string& appId,
                                          const std::string& instId, int pid) {
}

void RemoteSurfaceWatcher::OnWatcherRemoved(const std::string& appId,
                                            const std::string& instId,
                                            int pid) {
}

void RemoteSurfaceWatcher::OnWatcherChanged(const std::string& appId,
                                            const std::string& instId,
                                            int pid, const EvasObject& image) {
}

void RemoteSurfaceWatcher::SetChangedEventFilter(
    RemoteSurface::ChangedEventFilter filter) {
  for (auto& i : impl_->surfaces_) {
    (i.get())->SetChangedEventFilter(filter);
  }
}

int RemoteSurfaceWatcher::SetRemoteRender() {
  for (auto& i : impl_->surfaces_) {
    (i.get())->SetRemoteRender();
  }
  return 0;
}

}  // namespace screen_connector
